import { HttpsErrorCode } from './common';
import { deserialize, firebase, FirebaseApp, serialize } from '@nativescript/firebase-core';
let defaultFunctions;
const fb = firebase();
Object.defineProperty(fb, 'functions', {
    value: (app) => {
        if (!app) {
            if (!defaultFunctions) {
                defaultFunctions = new Functions();
            }
            return defaultFunctions;
        }
        return new Functions(app);
    },
    writable: false,
});
/**
  Firebase Functions Region - Region for which to run HttpsCallable method
  Set parameter using firebase().app().functions(regionOrCustomDomain: string)
  @link https://firebase.google.com/docs/reference/node/firebase.app.App
  @link https://firebase.google.com/docs/reference/ios/firebasefunctions/api/reference/Classes/FIRFunctions
  @note If not set, default region is used ('us-central1')
  */
let defaultRegionOrCustomDomain;
/**
  Add 'functions' method to FirebaseApp class
  @param regionOrCustomDomain(string)(Optional): Name of the Region or Custom Domain for which to Functions results
  @return Functions
  @see FirebaseFunctions
  @link https://firebase.google.com/docs/reference/ios/firebasefunctions/api/reference/Classes/FIRFunctions
  @see Supported Regions
  @see https://firebase.google.com/docs/functions/locations
  */
const fbApp = FirebaseApp;
Object.defineProperty(fbApp.prototype, 'functions', {
    value: (regionOrCustomDomain) => {
        defaultRegionOrCustomDomain = regionOrCustomDomain;
        if (!defaultFunctions) {
            defaultFunctions = new Functions();
        }
        return defaultFunctions;
    },
    writable: false,
});
function errorToCode(error) {
    let code = HttpsErrorCode.UNKNOWN;
    switch (error.code) {
        case 0 /* FIRFunctionsErrorCode.OK */:
            code = HttpsErrorCode.OK;
            break;
        case 1 /* FIRFunctionsErrorCode.Cancelled */:
            code = HttpsErrorCode.CANCELLED;
            break;
        case 2 /* FIRFunctionsErrorCode.Unknown */:
            code = HttpsErrorCode.UNKNOWN;
            break;
        case 3 /* FIRFunctionsErrorCode.InvalidArgument */:
            code = HttpsErrorCode.INVALID_ARGUMENT;
            break;
        case 4 /* FIRFunctionsErrorCode.DeadlineExceeded */:
            code = HttpsErrorCode.DEADLINE_EXCEEDED;
            break;
        case 5 /* FIRFunctionsErrorCode.NotFound */:
            code = HttpsErrorCode.NOT_FOUND;
            break;
        case 6 /* FIRFunctionsErrorCode.AlreadyExists */:
            code = HttpsErrorCode.ALREADY_EXISTS;
            break;
        case 7 /* FIRFunctionsErrorCode.PermissionDenied */:
            code = HttpsErrorCode.PERMISSION_DENIED;
            break;
        case 8 /* FIRFunctionsErrorCode.ResourceExhausted */:
            code = HttpsErrorCode.RESOURCE_EXHAUSTED;
            break;
        case 9 /* FIRFunctionsErrorCode.FailedPrecondition */:
            code = HttpsErrorCode.FAILED_PRECONDITION;
            break;
        case 10 /* FIRFunctionsErrorCode.Aborted */:
            code = HttpsErrorCode.ABORTED;
            break;
        case 11 /* FIRFunctionsErrorCode.OutOfRange */:
            code = HttpsErrorCode.OUT_OF_RANGE;
            break;
        case 12 /* FIRFunctionsErrorCode.Unimplemented */:
            code = HttpsErrorCode.UNIMPLEMENTED;
            break;
        case 13 /* FIRFunctionsErrorCode.Internal */:
            code = HttpsErrorCode.INTERNAL;
            break;
        case 14 /* FIRFunctionsErrorCode.Unavailable */:
            code = HttpsErrorCode.UNAVAILABLE;
            break;
        case 15 /* FIRFunctionsErrorCode.DataLoss */:
            code = HttpsErrorCode.DATA_LOSS;
            break;
        case 16 /* FIRFunctionsErrorCode.Unauthenticated */:
            code = HttpsErrorCode.UNAUTHENTICATED;
            break;
        default:
            break;
    }
    return code;
}
const FIRFunctionsErrorDomain = 'com.firebase.functions';
const FIRFunctionsErrorDetailsKey = 'details';
export class HttpsError extends Error {
    constructor(code, message, details = null, native = null) {
        super(message);
        this.code = code;
        this.details = details;
        this.native = native;
    }
}
function toHttpsError(error) {
    let details = null;
    if (error.domain == FIRFunctionsErrorDomain) {
        details = error.userInfo[FIRFunctionsErrorDetailsKey];
    }
    return new HttpsError(errorToCode(error), error.localizedDescription, details, error);
}
export class Functions {
    constructor(app) {
        if (app?.native) {
            if (defaultRegionOrCustomDomain) {
                this._native = isRegion(defaultRegionOrCustomDomain) // Check whether a Region has been set
                    ? FIRFunctions.functionsForAppRegion(app.native, defaultRegionOrCustomDomain) // @see https://firebase.google.com/docs/reference/ios/firebasefunctions/api/reference/Classes/FIRFunctions_+functionsforapp:region:
                    : isCustomDomain(defaultRegionOrCustomDomain) // Check whether using a Custom Domain has been set
                        ? FIRFunctions.functionsForAppCustomDomain(app.native, defaultRegionOrCustomDomain) // @see https://firebase.google.com/docs/reference/ios/firebasefunctions/api/reference/Classes/FIRFunctions_+functionsforapp:customdomain:
                        : FIRFunctions.functionsForApp(app.native); // @see https://firebase.google.com/docs/reference/ios/firebasefunctions/api/reference/Classes/FIRFunctions_+functionsforapp:
            }
            else {
                this._native = FIRFunctions.functionsForApp(app.native); // @see https://firebase.google.com/docs/reference/ios/firebasefunctions/api/reference/Classes/FIRFunctions_+functionsforapp:
            }
        }
        else {
            if (defaultFunctions) {
                return defaultFunctions;
            }
            defaultFunctions = this;
            if (defaultRegionOrCustomDomain) {
                this._native = isRegion(defaultRegionOrCustomDomain) // Check whether a Region has been set
                    ? FIRFunctions.functionsForRegion(defaultRegionOrCustomDomain) // @see https://firebase.google.com/docs/reference/ios/firebasefunctions/api/reference/Classes/FIRFunctions_+functionsforregion:
                    : isCustomDomain(defaultRegionOrCustomDomain) // Check whether using a Custom Domain has been set
                        ? FIRFunctions.functionsForCustomDomain(defaultRegionOrCustomDomain) // @see https://firebase.google.com/docs/reference/ios/firebasefunctions/api/reference/Classes/FIRFunctions_+functionsforcustomdomain:
                        : FIRFunctions.functions(); // @see https://firebase.google.com/docs/reference/ios/firebasefunctions/api/reference/Classes/FIRFunctions_+functions
            }
            else {
                this._native = FIRFunctions.functions(); // @see https://firebase.google.com/docs/reference/ios/firebasefunctions/api/reference/Classes/FIRFunctions_+functions
            }
        }
    }
    httpsCallable(name, options) {
        const callable = this.native.HTTPSCallableWithName(name);
        if (typeof options?.timeout === 'number') {
            callable.timeoutInterval = options.timeout;
        }
        return (data) => {
            return new Promise((resolve, reject) => {
                if (data) {
                    callable.callWithObjectCompletion(serialize(data), (result, error) => {
                        if (error) {
                            reject(toHttpsError(error));
                        }
                        else {
                            resolve(deserialize(result.data));
                        }
                    });
                }
                else {
                    callable.callWithCompletion((result, error) => {
                        if (error) {
                            reject(toHttpsError(error));
                        }
                        else {
                            resolve(deserialize(result.data));
                        }
                    });
                }
            });
        };
    }
    useEmulator(host, port) {
        this.native.useEmulatorWithHostPort(host, port);
    }
    get native() {
        return this._native;
    }
    get ios() {
        return this.native;
    }
    get app() {
        if (!this._app) {
            // @ts-ignore
            this._app = FirebaseApp.fromNative(this.native.app);
        }
        return this._app;
    }
}
/**
  Check whether a regionOrCustomDomain string is a Region for the http trigger, such as “us-central1”.
  @param regionOrCustomDomain(string): Text to parse
  @return boolean: TRUE if a Region; FALSE if not
  */
function isRegion(regionOrCustomDomain) {
    const elems = regionOrCustomDomain.split('.');
    return elems.length === 1 ? true : false;
}
/**
  Check whether a regionOrCustomDomain string is a Custom Domain for the http trigger, such as “https://mydomain.com”
  @param regionOrCustomDomain(string): Text to parse
  @return boolean: TRUE if a CustomDomain; FALSE if not
  */
function isCustomDomain(regionOrCustomDomain) {
    return !isRegion(regionOrCustomDomain);
}
//# sourceMappingURL=index.ios.js.map